use std::path::PathBuf;

use clap::Parser;
use tokio::runtime::Builder;

mod server;

pub const CLAP_STYLING: clap::builder::styling::Styles = clap::builder::styling::Styles::styled()
    .header(clap_cargo::style::HEADER)
    .usage(clap_cargo::style::USAGE)
    .literal(clap_cargo::style::LITERAL)
    .placeholder(clap_cargo::style::PLACEHOLDER)
    .error(clap_cargo::style::ERROR)
    .valid(clap_cargo::style::VALID)
    .invalid(clap_cargo::style::INVALID);

#[derive(Parser, Debug)]
#[command(name = "cargo")]
#[command(bin_name = "cargo")]
#[command(styles = CLAP_STYLING)]
enum Cli {
    Spiel(CliArgs),
}

#[derive(clap::Args, Debug)]
#[command(version, about, long_about = None)]
struct CliArgs {
    /// Directory of the Cargo project
    pub project: Option<PathBuf>,
}

fn main() {
    let Cli::Spiel(args) = Cli::parse();
    let runtime = Builder::new_current_thread()
        .worker_threads(2)
        .enable_io()
        .build()
        .unwrap();

    let project_path = if let Some(custom_path) = args.project {
        custom_path
    } else {
        std::env::current_dir().unwrap()
    };
    println!(
        "{:>12} Starting server in {}...",
        "spiel",
        project_path.to_str().unwrap()
    );
    runtime.block_on(server::start(project_path));
}
